# Wetter
# by De Kus <DeKus@gmx.net>
#
# Script for the weather page www.wetter.com.
# further documentation and the complete script output in German,
# because its a German site. All data from www.wetter.com served by
# the script are under the copyright of Wetter.com AG.
#
# Eine kleine Hilfe erhaltet ihr, wenn ihr den Wetter Befehl mit -hilfe aufruft.
# Weiterhin beachtet beim Editieren der Datei, dass diese Spalten bis 1000 Zeichen enthllt, was evtl.
# nicht jeder Editor untersttzt. Die Datei ist mit DOS-Zeilenumbrchen gespeichert, was jedoch von
# TCL i.d.R automatisch beim Einlesen in UNIX Zeilenbrche konvertiert wird.
#
# Wichtiger Hinweis: Falls die Copyright Hinweise zu Wetter.com AG entfernt werden sollen, msst 
# ------------------ ihr hierzu zuerst euch bei bei woys@wetter.com melden und um eine offizielle
#                    Erlaubnis zur Abschaltung jener bitten!
#
# 0.13.3  29.02.2008 Referrer Location gefixt
# 0.13.2  06.06.2007 Anpassung des regulren Ausdruck fr neue Vorschau
# 0.13.1  10.09.2006 Einige Vernderungen der Fehlererkennung von egghttp
#                    Fixing einiger mit der nderung angefallenen Fehlern
# 0.13    09.09.2006 Neuintegration von egghttp mit modifizierter Version
# 0.12.10 21.07.2006 Kleine Verbesserung der Ortssuche von Tsukasa
# 0.12.9  18.07.2006 berprfung, ob Lynx tatschlich verfgbar ist, bevor es verwendet wird
# 0.12.8  14.07.2006 DN statt IP, da die Seite es nun verlangt
# 0.12.7  12.07.2006 Anzeige des Ortes, der nicht gefunden wurde
# 0.12.6  07.07.2006 kleiner Sytanx Fehler gefixt und die Trennlinie als Variable gesetzt
# 0.12.5  06.07.2006 zwinge read wieder bis zu 64kb oder EOF zu leasen
# 0.12.4  28.06.2006 flags -|- fr langes Format nun auch fr unregistrierte Benutzer
# 0.12.3  28.05.2006 verschiedene irrelevante Codeverschiebungen bei DNS und useragent
# 0.12.2  16.05.2006 Wechsel in den syncronen Modus zum Lesen von Lynx, um immer den kompletten Code zu erhalten
# 0.12.1  10.05.2006 Wiederaufnahme des Versuchs mit externem Aufruf mit Lynx
# 0.12    29.03.2006 Hinzufgung von externen HTTP Aufrufen fr hhere Geschwindigkeit BETA
# 0.11    24.03.2006 nderung des Langformates des aktuellen Wetters (altes Format benutzbar)
#                    langes Format als Standard konfigurierbar
# 0.10.1  16.03.2006 Abschluss der Optimierung bei Vorschau
# 0.10    15.03.2006 Beginn der Optimieren durch Krzung des Body mit string first und range
#                    zurck auf http package
#
# Verffentlich unter der GNU GPL, siehe Datei Ende fr nheres

#
# Konfiguration
#

# System zum Downloaden der HTML Seite:
# 0: egghttp >=2.0 (inoffizielle Version)
# 1: http Packet von TCL
# 2: lynx (bentigt ebenfalls HTTP; ggf. wetter(lynx) anpassen)
set wetter(browser) 0

# maximale Antworten pro 5 Minuten (timer 5)
set wetter(floodmax) 3

# Datei in der die Ortsuchen gespeichert werden. Beschleunigt die Ausgabe und
# reduziert Traffik etc. Falls es einen ungnstigen Eintrag gibt, kann sie per
# Hand bearbeitet werden, da die Datei fr jede Anfrage neu geladen wird.
set wetter(cachedatei) "wetter.cache"

# Leerzeichen getrennte Liste von Channels in denen !wetter nicht gehen soll
set wetter(nichtdort) ""

# Flags welche die -lang Option im Channel benutzen drfen (Query drfen immer alle). "-|-" fr alle
set wetter(langflags) "ol|ol"

# Flags welche auch wrend nach Erreichen von Floodmax noch Anfordern drfen, jedoch auch nicht wrend eines
# anderen Aufrufs (oder eben nach einem TCL Fehler).
# Hinweis: das Skript msste umgeschrieben werden, um mehrere Aufrufe parallel ausfhren zu knnen.
set wetter(keinflood) "of|of"

# Der Useragent welcher der Seite gesendet wird. Sollte relativ egal sein, ich mags eben ehrlich.
# Es wird versucht das Betriebsystem anzuhngen
set wetter(useragent) "Tcl http client (TCL [info patchlevel]; http [package require http 2.4]; de-DE; "

# neues aktuelle Wetterformat ein/aus
set wetter(langformat) 1

# Trennlinie im Langenformat. Einige Schrifftarten zeigen rechteckige Kstchen
# statt einer gerade Linie (nur im neuen Format)
set wetter(linie) "\002\002"

# langes Format als Standard ein/aus
set wetter(langstd) 0

# Gib die Befehlszeile fr Lynx an. Nur ntig, wenn extern gerufen wird.
# Wenn Lynx nicht im Suchpfad ist fge den absoluten oder relativen Pfad hinzu.
# Sollte Lynx mit der Zeile nicht zu starten sein, wird es deaktiviert.
set wetter(lynx) "lynx -noprint -source -noredir -nostatus"

# Einstellungen fr bind pub & msg
set wetter_cmd "!wetter"
set wetter_flag "-"

#
# Begin Code
#

if {$wetter(browser) == 2} {
	if {[catch {eval exec $wetter(lynx)} err]} {
		set wetter(browser) 1
		putlog "Wetter: \[Hinweis\] Konnte Lynx nicht starten (falscher Pfad, keine Berechtigung oder nicht installiert), auf HTTP Package umgeschaltet."
	}
}
if {$wetter(browser) == 1} {
# Dies ist eher ein geschtzte Vorraussetzung
	package require http 2.4
} elseif {$wetter(browser) == 0} {
	if {![info exists egghttp(version)]} {
		die "Fehler: Konnte keine egghttp.tcl fr wetter.tcl geladen finden, du solltest dies ndern!"
	} elseif {[string range $egghttp(version) 0 0] < 2} {
		putlog "wetter.tcl: Deine egghttp.tcl scheint veraltet zu sein, Version 2.0.0 oder hher erforderlich, da es sonst ziemlich sicher Fehler gibt!"
	}
} elseif {$wetter(browser) != 2} {
	die "Fehler: '$wetter(browser)' ist kein gltiger Browsertype in wetter(browser) fr wetter.tcl!"
}

bind pub $wetter_flag $wetter_cmd getwetter_cmd
bind msg $wetter_flag $wetter_cmd getwetter_msgcmd
bind dcc m wetterreset wetterreset

# warum diese Variablen behalten? Sinnlos
unset wetter_cmd
unset wetter_flag

# Floodschutz Variablen Initialisierung
set wetter(flood) 0
set wetter(inuse) 0
set wetter(lange) 0
set wetter(debug) 0

# Anlegung der regulren Ausdrcke als globalen Array, um sie effektiver zu cachen.
# URL Findung - Weiterleitung
set re(wetter1) {&type=([^&]+)&?.*?&id=([0-9]+)}
# URL Findung - Liste
set re(wetter2) {&type=([^&]+)&?.*?&id=([0-9]+)"}
# re(wetter3) entfllt
# Ortname
set re(wetter4) {>([A-Za-z'\- ().]+)}
# aktuelles Wetter
set re(wetter5) {"border-left: none;">(.+?)</td>\n.+"HeadlineXL">(.+?) &deg;C.+"border-left: none;">.+?"border-left: none;">(.+?)</td>\n.+?"border-left: none;">(.+?)</td>\n.+?"border-left: none;">(.+?)</td>\n.+?"border-left: none;">(.+?)</td>\n.+?"border-left: none;">(.+?)</td>\n.+"border-left: none;">(.+?)</td>\n.+?"border-left: none;">(.+?)</td>\n}
# Wetterwarnungen
set re(wetter6) {\t<.{15}yWhite">([^\n]+?)<b}
# Vorhersage
set re(wetter7) {="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %).+="line-height: 15px;">.{7}(.+?)<br>.+?\t(-?[0-9]{1,2} &deg;C)<br>\t{6}([0-9]{1,3} %|k.A. %)}
# Min/Max
set re(wetter8) {="line-height: 20px;">(-?[0-9]{1,2} / -?[0-9]{1,2} &deg;C)<.+?="line-height: 20px;">(-?[0-9]{1,2} / -?[0-9]{1,2} &deg;C)<.+?="line-height: 20px;">(-?[0-9]{1,2} / -?[0-9]{1,2} &deg;C)<}
# Warnlevel
set re(wetter9) {img/www/buttons/warnings_level([0-9]?).gif}
# Numerischer Code
set re(http) {([0-9]{3})}

# Anhngen des OS
if {![info exists wetter(useragent)]} {
	set wetter(useragent) ""
} elseif {[file executable /bin/uname]} {
# sollte unter BSD/Linux/Cygwin etc. funktionieren
	append wetter(useragent) "[exec /bin/uname -sr])"
} elseif {[file executable uname]} {
	append wetter(useragent) "[exec uname -sr])"
} elseif {[info exists env(OS)]} {
# sollte unter Windows ohne Cygwin Installation funktionieren
	append wetter(useragent) "$env(OS))"
} else {
	append wetter(useragent) "Unix compatible)"
}

set wetter(server) {www.wetter.com}
# damit umgehen wir die Benutzung syncrone Nutzung von gethostbyname durch SOCKET in http
proc getwetterdns:cb {ip host status} {
	set ::wetter(ip) $ip
}
dnslookup $wetter(server) getwetterdns:cb

if {![file exists $wetter(cachedatei)]} {
	if {[catch {set wetterdatei [open $wetter(cachedatei) w]} err]} {
		if {[string match /* $wetter(cachedatei)] || [string match ~* $wetter(cachedatei)]} {
			set wetterdatei $wetter(cachedatei)
		} else {
			set wetterdatei "[pwd]/$wetter(cachedatei)"
		}
		putlog "Wetter: \[Warnung\] Kann Datei '$wetterdatei' nicht zum Schreiben ffnen: $err"
	} else {
		putlog "Wetter: \[Hinweis\] Cachedatei '$wetter(cachedatei)' angelegt."
	}
	close $wetterdatei
	if {[info exists err]} {unset err}
	unset wetterdatei
}

proc getwetter_msgcmd {nick uhost hand text} {
	getwetter_cmd $nick $uhost $hand $nick $text
	return 1
}

proc getwetter_cmd {nick uhost hand chan text} {
	global wetter re
	set text [split $text]
	set param(lang) $wetter(langstd)
	set param(hilfe) 0
	set param(vorschau) 0
	set param(neuerstandard) 0
	if {$nick == $chan} {
		set query 1
	} else {
		set query 0
	}
	while {[string match -* [lindex $text 0]]} {
		switch -glob -- [string trimleft [lindex $text 0] -] {
			l*			{
				if {[matchattr $hand $wetter(langflags) $chan] || $wetter(langflags) == "-|-" || $query} {
					set param(lang) 1
				}
			}
			k*			{set param(lang) 0}
			heu*		-
			v			-
			vorschau	-
			v*1		{set param(vorschau) 1}
			m*			-
			v*2		{set param(vorschau) 2}
			*			-
			v*3 		{set param(vorschau) 3}
			s*			{
				if {$hand == "*"} {
					puthelp "NOTICE $nick :Ich erkenne dich nicht wieder, du kannst so keinen Standard definieren oder benutzen."
				} else {
					set param(neuerstandard) 1
				}
			}
			default	{set param(hilfe) 1}
		}
		set text [lreplace $text 0 0]
	}
	if {$param(hilfe)} {
		puthelp "NOTICE $nick :Das Wetter wird von www.wetter.com abgerufen, dies dauert gewhnlich einige Sekunden. Mgliche Parameter (Anfangsbuchstaben gengen) sind:"
		if {$wetter(langstd)} {
			puthelp "NOTICE $nick : \002 -kurz \002\002 \002\002 \002\002 \002\002 \002: kurzes, einzeiliges Ausgabeformat."
		} else {
			puthelp "NOTICE $nick : \002 -lang \002\002 \002\002 \002\002 \002\002 \002: extra langes Ausgabeformat ((half-)op oder Query Beschrnkung)."
		}
		puthelp "NOTICE $nick : \002 -vorschau \002: Gibt die Vorschau fr heute an."
		puthelp "NOTICE $nick : \002 -vorschau2\002: Gibt die Vorschau fr morgen an."
		puthelp "NOTICE $nick : \002 -vorschau3\002: Gibt die Vorschau fr bermorgen an."
		puthelp "NOTICE $nick : \002 -standard \002: Speichert eine neue Standard Wetterausgabe."
		puthelp "NOTICE $nick :Den gewnschten Ort schreibt ihr einfach ganz hinten hin als Name oder PLZ, wahlweise knnt ihr mit Komma das Bundesland anhngen. Bitte darauf achten, dass automatisch das erste Ergbnis gewhlt wird!"
	} elseif {[lsearch -exact $wetter(nichtdort) $chan] != -1} {
		puthelp "NOTICE $nick :Bitte benutze den Befehl in einem anderen Channel oder im Query"
	} elseif {$wetter(inuse) == 1 || ($wetter(flood) >= $wetter(floodmax) && ![matchattr $hand $wetter(keinflood) $chan] && $query)} {
		puthelp "NOTICE $nick :Floodschutz aktiv, bitte warte einige Minuten. Falls sich nichts ndert, trat wahrscheinlich vorher ein TCL Fehler auf."
	} elseif {$text == ""} {
		if {$hand == "*"} {
			puthelp "NOTICE $nick :Du musst einen Ort angeben, fr Hilfe gib als Paremeter -hilfe an."
		} elseif {[set data [getuser $hand XTRA wetter]] == {}} {
			puthelp "NOTICE $nick :Ich habe keine Standard Wetterausgabe fr dich gespeichert, daher gib bitte einen Ort an."
		} else {
			set wetter(ort) [lindex $data 0]
			set wetter(type) [lindex $data 1]
			set wetter(vorschau) [lindex $data 2]
			set wetter(loc) "Standardeinstellung"
			set wetter(loc2) {}
			set wetter(lang) $param(lang)
			set wetter(inuse) 1
			set wetter(error) 0
			set wetter(ns) 0
			set wetter(chan) $chan
			set wetter(nick) $nick
			if {!$query} {
				incr wetter(flood)
				timer 5 wetterflood
			}
			getwetter
			return 1
		}
	} else {
		set wetter(loc) [string trim [lindex [split $text ,] 0]]
		set wetter(loc2) [string trim [lindex [split $text ,] 1]]
		set wetter(vorschau) $param(vorschau)
		set wetter(lang) $param(lang)
		set wetter(inuse) 1
		set wetter(chan) $chan
		set wetter(nick) $nick
		set wetter(ns) $param(neuerstandard)
		set wetter(error) 0
		if {!$query} {
			incr wetter(flood)
			timer 5 wetterflood
		}
		getwetterloc
		return 1
	}
	return 0
}

proc getwetterloc "" {
	global wetter
	set ort ""
	set wetterdatei [open $wetter(cachedatei) r]
		set wettercache [gets $wetterdatei]
	close $wetterdatei
	if {[catch {llength $wettercache} err]} {
		putlog "Wetter: \[Warnung\] keine gltige Liste in der Wettercache Datei. Dieser Fehler ist permanent, behebbar durch Fachkenntnis (Liste fixen) oder Lschen. Das Skript ld nun alle Orte von der Webseite, bis dies behoben wurde."
	} else {
		foreach entry $wettercache {
			foreach {id mode nummer name ext} $entry {
				if {$nummer == $wetter(loc) || [string equal -nocase $name $wetter(loc)] && [string match -nocase *$wetter(loc2)* $ext]} {
					if {[string is integer -strict $id]} {
						set ort $id
						set type $mode
					} else {
						puthelp "NOTICE $wetter(nick) :Fehler in der Cache Datei, erwartete einen Integer aber erhielt '$id'. Benutze bitte den DCC Befehl wettercacheupdate oder repariere/lsche den Cache manuel."
						set wetter(error) 1
					}
					break
				}
			}
		}
	}
	if {$ort == ""} {
		set url "/v2/?SID=&LANG=DE&LOC=7004&LOCFROM=0001&LOCFULL=7000&x=0&y=0&search=$wetter(loc)"
		switch $wetter(browser) {
			0	{
				if {[catch {egghttp:geturl "http://$wetter(ip)$url" getwetterloc:cb -host $wetter(server)} err] } {
					puthelp "NOTICE $wetter(nick) :Fehler '$err' beim ffnen der HTTP Verbindung zu '$wetter(ip)'. Homepage oder Domain Auflsung down?!"
					set wetter(error) 1
					return 1
				}
			}
			1	-
			2	{
				if {$wetter(useragent) != {}} {
					::http::config -useragent $wetter(useragent)
				}
				if {[catch {::http::geturl "http://$wetter(server)$url" -headers {Connection close} -command getwetterloc:cb -blocksize 65532} err]} {
					puthelp "NOTICE $wetter(nick) :Fehler '$err' beim ffnen der HTTP Verbindung zu '$wetter(server)'. Homepage oder Domain Auflsung down?!"
					set wetter(error) 1
					return 1
				}
			}
		}
	} else {
		set wetter(ort) $ort
		set wetter(type) $type
		if {$wetter(ns)} {
			setuser [nick2hand $wetter(nick)] XTRA wetter [list $ort $type $wetter(vorschau)]
			switch $wetter(vorschau) {
				0	{set tag "Aktuelles Wetter"}
				1	{set tag "Heutige Vorhersage"}
				2	{set tag "Morgige Vorhersage"}
				3	{set tag "bermorgige Vorhersage"}
			}
			puthelp "NOTICE $wetter(nick) :Neuer Wetterstandard: $tag von $wetter(loc)"
		}
		getwetter
	}
	return 0
}

proc getwetterloc:cb {token} {
	global wetter re
	set ort {}
	set name {}
	set type {}
	set ext1 {}
	set ext2 {}
	switch $wetter(browser) {
		0	{
			set state(body) [egghttp:data $token]
			set state(ncode) [egghttp:code $token]
		}
		1	-
		2	{
			upvar #0 $token state
			set state(ncode) [::http::ncode $token]
		}
	}
	if {$wetter(debug)} {
		set debugDatei [open {loc.txt} w]
			puts -nonewline $debugDatei [array get state]
		close $debugDatei
	}
	if {$state(ncode) == 302} {
		switch $wetter(browser) {
			0	{
				array set headers [egghttp:headers $token]
				set location $headers(location)
				if {$wetter(debug)} {
					set debugDatei [open {loc-h.txt} w]
						puts -nonewline $debugDatei [array get headers]
					close $debugDatei
				}
			}
			1	-
			2	{
				array set meta $state(meta)
				set location $meta(Location)
				if {$wetter(debug)} {
					set debugDatei [open {loc-h.txt} w]
						puts -nonewline $debugDatei $state(meta)
					close $debugDatei
				}
			}
		}
		regexp $re(wetter1) $location {} type ort
#		if {$wetter(debug)} {
#			putdcc [hand2idx [lindex $::owner 0]] "$location"
#			putdcc [hand2idx [lindex $::owner 0]] "T: $type; O: $ort"
#		}
	} else {
		set i [string first {<ul class="Body">} $state(body)]
		set j [string first {</ul>} $state(body) $i]
		set links [string range $state(body) $i $j]
# Vorrbergehende Verbesserung der Ortsuche von Tsukasa
		set i [string first {<li>} $links]
		set j [string first {</li>} $links $i]
		set links [string range $links $i $j]
		regexp $re(wetter2) $links "" type ort
#		set re(wetter3) "(?i)\042 class=\042Headline\042>(.{2,30})</a>,(.{1,30})$wetter(loc2)(.{1,30})\\050(.{3,6})\\051\t"
		regexp "(?i)\042 class=\042Headline\042>(.{2,30})</a>,(.{1,30})$wetter(loc2)(.{1,30})\\050(.{3,6})\\051\t" \
				$links "" name ext1 ext2 nummer
		set ext [string trim [lindex [split "$ext1$wetter(loc2)$ext2" ","] 0]]
	}
	if {$ort != ""} {
		set wetterdatei [open $wetter(cachedatei) r+]
			set wettercache [gets $wetterdatei]
			if {$name == ""} {
				if {[string is integer $wetter(loc)]} {
					lappend wettercache [list $ort $type $wetter(loc) . .]
				} else {
					lappend wettercache [list $ort $type . $wetter(loc) .]
				}
			} else {
				lappend wettercache [list $ort $type $nummer $name $ext]
			}
			seek $wetterdatei 0
			puts -nonewline $wetterdatei $wettercache
		close $wetterdatei
		set wetter(ort) $ort
		set wetter(type) $type
		if {$wetter(ns)} {
			setuser [nick2hand $wetter(nick)] XTRA wetter [list $ort $type $wetter(vorschau)]
			switch $wetter(vorschau) {
				0	{set tag {Aktuelles Wetter}}
				1	{set tag {Heutige Vorhersage}}
				2	{set tag {Morige Vorhersage}}
				3	{set tag {bermorige VOrhersage}}
			}
			puthelp "NOTICE $wetter(nick) :Neuer Wetterstandard: $tag von $wetter(ort)"
		}
		getwetter
	} else {
		if {$wetter(loc2) != ""} {
			set ort ", $wetter(loc2)"
		}
		puthelp "NOTICE $wetter(nick) :Fehler, kann den Ort '$wetter(loc)$ort' auf www.wetter.com nicht finden."
		wetterCleanup $token
		wetterquit
		return 1
	}
	wetterCleanup $token
	return 0
}

proc getwetter {} {
	global wetter
	if {$wetter(vorschau) == 0} {
		set url "/v2/?SID=&LANG=DE&type=$wetter(type)&LOC=7001&LOCFROM=7000&id=$wetter(ort)"
	} else {
		set url "/v2/?SID=&LANG=DE&type=$wetter(type)&LOC=7000&LOCFROM=7000&id=$wetter(ort)"
	}
	switch $wetter(browser) {
		0	{
			if {[catch {egghttp:geturl "http://$wetter(ip)$url" getwetter:cb -host $wetter(server)} err] } {
				puthelp "NOTICE $wetter(nick) :Fehler '$err' beim ffnen der HTTP Verbindung zu '$wetter(ip)'. Homepage oder Domain Auflsung down?!"
				set wetter(error) 1
				return 1
			}
		}
		1	{
			if {$wetter(useragent) != {}} {
				::http::config -useragent $wetter(useragent)
			}
			if {[catch {::http::geturl "http://$wetter(server)$url" -headers {Connection close} -command getwetter:cb -blocksize 65532} err]} {
				puthelp "NOTICE $wetter(nick) :Fehler '$err' beim ffnen der HTTP Verbindung zu '$wetter(server)'. Homepage down?!"
				set wetter(error) 1
				return 1
			}
		}
		2	{
			set fd [open "|$wetter(lynx) \"http://$wetter(server)$url\"" r]
			fconfigure $fd -blocking 0
			read $fd
			fileevent $fd readable [list getwetter:cb $fd]
		}
	}
	return 0
}

proc getwetter:cb {token} {
#	set time [clock clicks -milliseconds]
	global wetter re
	if {$wetter(error) == 0} {
		set name {}
		set ncode {}
		set ort [string trim "$wetter(loc), $wetter(loc2)" {, }]
		switch $wetter(browser) {
			0	{
				set state(body) [egghttp:data $token]
				set state(ncode) [egghttp:code $token]
				set state(http) [lindex [split [egghttp:headers $token] \n] 0]
				set state(error) [egghttp:errormsg $token]
			}
			1	{
				upvar #0 $token state
				regexp $re(http) $state(http) {} state(ncode)
			}
			2	{
				fconfigure $token -blocking 1
				array set state [list body [read $token 65535]]
				if {[catch {close $token} err]} {
					putlog "Wetter: \[Hinweis\] Fehler von Lynx: $err"
				}
				set state(ncode) 200
			}
		}
		if {$wetter(debug)} {
			set debugDatei [open {wetter.txt} w]
				puts -nonewline $debugDatei [array get state]
			close $debugDatei
		}
		set i [string first {class="HeadlineXL"} $state(body)]
		regexp $re(wetter4) [string range $state(body) $i [expr {$i + 50}]] {} name
		if {$wetter(browser) == 0 && $state(error) != "Ok"} {
			putlog "wetter.tcl: Fehler von egghttp '$state(error)'"
		}
		if {$state(ncode) == 500} {
			puthelp "PRIVMSG $wetter(chan) :Webserver meldet Statuscode 500. Ein Serverfehler verhindert vorrbergehend das Abrufen von wetter.com."
		} elseif {$state(ncode) != 200 && !($wetter(browser) == 0 && $state(ncode) == 0)} {
			puthelp "NOTICE $wetter(nick) :Fehler vom Webserver: $state(http)."
		} elseif {[string match "*Zur Zeit liegen leider keine aktuellen Wetterdaten vor*" $state(body)]} {
			puthelp "NOTICE $wetter(nick) :Fehler, es liegen keine aktuellen Wetterdaten auf www.wetter.com vor fr '$ort'."
		} elseif {$name == {}} {
			puthelp "NOTICE $wetter(nick) :Fehler beim Parsen der Wetterseite auf www.wetter.com fr '$ort'."
		} elseif {$wetter(vorschau) == 0} {
			set himmel {}
			set temp {}
			set winds {}
			set windr {}
			set windg {}
			set windb {}
			set feuchte {}
			set sicht {}
			set druck {}
			set i [string first {"border-left: none;">} $state(body)]
			set j [string first {</table>} $state(body) $i]
			regexp $re(wetter5) [string range $state(body) $i $j] {} himmel temp windr windg winds windb druck feuchte sicht
			set windn [lindex [split $windg] 0]
			if {[string is double -strict $temp] && [string is int -strict $windn] && [string is int -strict [set feucht [string map {{ %} {}} $feuchte]]]} {
				set windchill [expr {round((33+(0.478+0.237*sqrt($windn)-0.0124*$windn)*($temp-33))*10)/10.0}]
				set x [expr {1 - 0.01 * $feucht}]
				set gefuil [expr {$windchill + round(((0.5555)*((6.11 * exp(5417.7530 * ((1/273.16) - (1/($temp - ((14.55 + 0.114 * $temp)* $x + pow([expr (2.5 + 0.007 * $temp) * $x],3) + (15.9 + 0.117 * $temp) * pow($x,14)) + 273.15))))) - 10.0)) * 10) / 10.0}]
			} else {
				set gefuil ?
			}
			set himmel [string map {sonnig klar} $himmel]
			if {$wetter(lang) == 0} {
				set output "Es ist in $name [string map {. ,} $temp] ([string map {. ,} $gefuil]) C"
				switch -- $himmel {
					k.A.	{}
					default	{
						append output ", $himmel"
					}
				}
				if {$windr == {windstill}} {
					append output { und herrscht Windstille.}
				} elseif {$windr == {k.A.} && $winds == {k.A.}} {
					append output .
				} else {
					append output " und weht ein Wind der Strke [string map {k.A. unbekannt} [string trimr $winds { bft\n}]] aus [string map {k.A. unbekannt} $windr]."
				}
#				catch {putdcc [hand2idx De_Kus] $output}
# Copyright Hinweis ntig, da alle Wetter Daten dem Urheberrecht unterliegen
				puthelp "PRIVMSG $wetter(chan) :$output  Wetter.com AG"
#				set time6 [clock clicks -milliseconds]
				set i [string first {class="TAB_BOX_3_BOXED"} $state(body)]
				set j [string first {!-- SYNOPS START -->} $state(body) $j]
				set achtung [regexp -inline -all $re(wetter6) [string range $state(body) $i $j]]
#				putdcc 11 6:[expr [clock clicks -milliseconds] - $time6]ms
				set i [string first {img/www/buttons/warnings_level} $state(body)]
				set j [string first {<!-- END LEFT COLUMN -->} $state(body) $i]
				set warnlevel [regexp -inline -all $re(wetter9) [string range $state(body) $i $j]]
				set achtungl [llength $achtung]
				for {set i 1} {$i < $achtungl} {incr i 2} {
					if {[lindex $warnlevel $i] > 1} {
						puthelp "PRIVMSG $wetter(chan) :\002Unwetter Warnung:\002 [lindex $achtung $i]"
					}
				}
			} else {
# Copyright Hinweis ntig, da alle Wetter Daten dem Urheberrecht unterliegen
				puthelp "PRIVMSG $wetter(chan) :Detaillierte \002Wetterbersicht\002 fr $name: (Werte unterliegen Copyright  [clock format [unixtime] -format %Y] Wetter.com AG)"
				if {$wetter(langformat)} {
					puthelp "PRIVMSG $wetter(chan) :$himmel $wetter(linie) [string map {. ,} $temp] ([string map {. ,} $gefuil]) C"
					puthelp "PRIVMSG $wetter(chan) :[string map {bft Beaufort} $winds] ([string map {k.A. unbekannt} $windg] mit Ben bis [string map {k.A. unbekannt} $windb]) aus $windr"
					puthelp "PRIVMSG $wetter(chan) :$druck $wetter(linie) $feuchte rel. Feuchte $wetter(linie) $sicht Sichtweite"
				} else {
					puthelp "PRIVMSG $wetter(chan) :\002Temperatur (gefhlt) & Wetter:\002 [string map {. ,} $temp] ([string map {. ,} $gefuil]) C, $himmel"
					puthelp "PRIVMSG $wetter(chan) :\002Wind:\002 [string map {bft Beaufort} $winds] ([string map {k.A. unbekannt} $windg] mit Ben bis [string map {k.A. unbekannt} $windb]) aus $windr"
					puthelp "PRIVMSG $wetter(chan) :\002Luftdruck & -feuchtigkeit:\002 $druck, $feuchte"
					puthelp "PRIVMSG $wetter(chan) :\002Sicht:\002 $sicht"
				}
				set i [string first {Achtung:<br>} $state(body)]
				set j [string first {</table>} $state(body) $i]
				set achtung [regexp -inline -all $re(wetter6) [string first $state(body) $i $j]]
				set i [string first {img/www/buttons/warnings_level} $state(body)]
				set j [string first {<!-- END LEFT COLUMN -->} $state(body) $i]
				set warnlevel [regexp -inline -all $re(wetter9) [string range $state(body) $i $j]]
				set achtungl [llength $achtung]
				for {set i 1} {$i < $achtungl} {incr i 2} {
					switch [lindex $warnlevel $i] {
						2	{set warnung "\002Achtung, Warnung vor Unwetter:\002"}
						default	{set warnung "\002Warnung:\002"}
					}
					puthelp "PRIVMSG $wetter(chan) :$warnung [lindex $achtung $i]"
				}
			}
		} else {
#			set time7 [clock clicks -milliseconds]
			set i [string first {Wetterzustand:} $state(body)]
			set j [string first {<!-- FORECAST END -->} $state(body) $i]
			if {[regexp $re(wetter7) [string range $state(body) $i $j] {} himmel(1) temp(1) ns(1) himmel(2) temp(2) ns(2) himmel(3) temp(3) ns(3) himmel(4) temp(4) ns(4) himmel(5) temp(5) ns(5) himmel(6) temp(6) ns(6) himmel(7) temp(7) ns(7) himmel(8) temp(8) ns(8) himmel(9) temp(9) ns(9)]
				&& [regexp $re(wetter8) [string range $state(body) $i $j] {} minmax(1) minmax(2) minmax(3)]} {
#				putdcc 11 7:[expr [clock clicks -milliseconds] - $time7]ms
				switch $wetter(vorschau) {
					1	{
						set vor 1
						set tag "heute"
					}
					2	{
						set vor 4
						set tag "morgen"
					}
					3	{
						set vor 7
						set tag "bermorgen"
					}
				}
				if {$wetter(lang) == 0} {
					puthelp "PRIVMSG $wetter(chan) :In $name soll es laut Vorhersage fr $tag Mittag $himmel([expr $vor + 1]) werden, [string map {&deg; \260} $temp([expr $vor + 1])] haben und eine Chance auf Niederschlag von $ns([expr $vor + 1]) haben.  Wetter.com AG"
					set i [string first {Achtung:<br>} $state(body)]
					set j [string first {</table>} $state(body) $i]
					set achtung [regexp -inline -all $re(wetter6) [string first $state(body) $i $j]]
					set i [string first {img/www/buttons/warnings_level} $state(body)]
					set j [string first {<!-- END LEFT COLUMN -->} $state(body) $i]
					set warnlevel [regexp -inline -all $re(wetter9) [string range $state(body) $i $j]]
					set achtungl [llength $achtung]
					for {set i 1} {$i < $achtungl} {incr i 2} {
						if {[lindex $warnlevel $i] > 1} {
							puthelp "PRIVMSG $wetter(chan) :\002Unwetter Warnung:\002 [lindex $achtung $i]"
						}
					}
				} else {
#					putdcc 11 7:[expr [clock clicks -milliseconds] - $time7]ms
					puthelp "PRIVMSG $wetter(chan) :Detaillierte \002Wettervorhersage\002 fr $tag in $name: (Werte unterliegen Copyright  [clock format [unixtime] -format %Y] Wetter.com AG)"
					puthelp "PRIVMSG $wetter(chan) :\002Morgens:\002 [string map {&deg; \260 . ,} $temp($vor)], $himmel($vor), Niederschlag: $ns($vor)"
					puthelp "PRIVMSG $wetter(chan) :\002Mittags:\002 [string map {&deg; \260 . ,} $temp([expr $vor + 1])], $himmel([expr $vor + 1]), Niederschlag: $ns([expr $vor + 1])"
					puthelp "PRIVMSG $wetter(chan) :\002Abends :\002 [string map {&deg; \260 . ,} $temp([expr $vor + 2])], $himmel([expr $vor + 2]), Niederschlag: $ns([expr $vor + 2])"
					puthelp "PRIVMSG $wetter(chan) :\002Tempatur- Tief/Hoch:\002 [string map {&deg; \260 . ,} $minmax($wetter(vorschau))]"
					if {$wetter(vorschau) == 1} {
						set i [string first {Achtung:<br>} $state(body)]
						set j [string first {</table>} $state(body) $i]
						set achtung [regexp -inline -all $re(wetter6) [string first $state(body) $i $j]]
						set i [string first {img/www/buttons/warnings_level} $state(body)]
						set j [string first {<!-- END LEFT COLUMN -->} $state(body) $i]
						set warnlevel [regexp -inline -all $re(wetter9) [string range $state(body) $i $j]]
						set achtungl [llength $achtung]
						for {set i 1} {$i < $achtungl} {incr i 2} {
							switch [lindex $warnlevel $i] {
								2	{set warnung "\002Achtung, Warnung vor Unwetter:\002"}
								default	{set warnung "\002Warnung:\002"}
							}
							puthelp "PRIVMSG $wetter(chan) :$warnung [lindex $achtung $i]"
						}
					}
				}
			} else {
				puthelp "NOTICE $wetter(nick) :Fehler beim Parsen der Wetterdaten auf www.wetter.com fr '$ort'. Dieser Fehler ist wahrscheinlich anhaltend, solange sich die Wetterdaten nicht ndern. Bitte melde diesen Fehler mit genauer Ortangabe."
			}
		}
	}
	wetterCleanup $token
	wetterquit
#	putlog "e:[expr [clock clicks -milliseconds] - $time]ms"
	return 0
}

proc wetterflood {} {
	global wetter
	if {$wetter(flood) <= 1} {
		set wetter(flood) 0
	} else {
		incr wetter(flood) -1
	}
	return 0
}

proc wetterreset {hand idx text} {
	putdcc $idx {Zeige letzten TCL Fehler:}
	putdcc $idx $::errorInfo
	wetterquit
	return 1
}

proc wetterCleanup {token} {
	switch $::wetter(browser) {
		0	{egghttp:cleanup $token}
		1	-
		2	{::http::cleanup $token}
	}
}

proc wetterquit {} {
	global wetter
# Keine Ahnung, ob das langsamer wie inline Code ist, aber sieht bersichtlicher aus :)
	foreach var {chan lang vorschau nick loc loc2 ort type error ns} {
# Mit der IF vermeiden wir unntige Erzeugung von TCL Fehlern, die $errorInfo berschreiben
		if {[info exists wetter($var)]} {
			unset wetter($var)
		}
	}
	set wetter(inuse) 0
	set wetter(lange) 0
	return 0
}

putlog {Wetter.tcl v0.13.3 geladen!}

# Copyright  2005-2008 De Kus
# This TCL script (Wetter) for the eggdrop IRC bot is released
# under the GNU GPL, refer GPL.txt included in the archive.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
